(import [os]
        [sys]
        [signal]
        [StringIO [StringIO]]
        [json]
        [hy.cmdline [HyREPL]]
        [hy]
        [uuid [uuid4]]
        [flask [Flask request make_response render_template send_from_directory]]
        [math [pi]]
        [cairocffi :as cairo])

(defclass MyHyREPL [HyREPL]
  [[_eval (fn [self code]
           (setv old-stdout sys.stdout)
           (setv old-stderr sys.stderr)
           (setv fake-stdout (StringIO))
           (setv sys.stdout fake-stdout)
           (setv fake-stderr (StringIO))
           (setv sys.stderr fake-stderr)
           (HyREPL.runsource self code "<input>" "single")
           (setv sys.stdout old-stdout)
           (setv sys.stderr old-stderr)
           {"stdout" (fake-stdout.getvalue) "stderr" (fake-stderr.getvalue)})]])

(require y-turtle)

(defn draw [env last-inst]
  (let [[repl (MyHyREPL)]
        [session-id (get-session-id)]
        [svg-file (.format "svg/{}.svg" session-id)]
        [init-code ["(import [cairocffi :as cairo])"
                    "(import [math [pi]])"
                    "(require y-turtle)"
                    (.format "(setv ps (cairo.SVGSurface \"{}\" 200 200))"
                             svg-file)
                    "(setv cr (cairo.Context ps))"
                    "(.translate cr 100 100)"
                    "(.move_to cr 0 0)"
                    "(color 0 1 0)"
                    "(line 1)"]]
        [all-code (+ init-code
                     env
                     [last-inst])]
        [code-string (.join "\n" all-code)]]
    (print code-string)

    (defn timeout-handler [signum frame]
      (raise Exception))

    (signal.signal signal.SIGALRM timeout-handler)
    (signal.alarm 2)

    (try
      (print (repl._eval code-string))
      (except [Exception e]
        (print e))
      (finally
        (signal.alarm 0)))

    (json.dumps "")  ; output on the client
    ))

(defn get-session-id []
  (defn get-id [] (.get request.cookies "session-cookie"))
  (defn new-id [] (let [[id* (uuid4)]] id*.hex))
  (if (get-id) (get-id) (new-id)))

(def app (Flask __name__))

(with-decorator (apply app.route ["/"] {"methods" ["GET"]})
  (fn []
    (setv session-id (get-session-id))
    (setv svg-file (.format "svg/{}.svg" session-id))
    (setv response (make_response
                     (apply render_template
                       ["index.html"]
                       {"svg_file" svg-file})))
    (print session-id)
    (.set_cookie response "session-cookie" session-id)
    (draw [] "")
    response))

(with-decorator (apply app.route ["/eval"] {"methods" ["POST"]})
  (fn [] 
    (let [[input (request.get_json)]
          [env (get input "env")]
          [code (get input "code")]]
      (draw env code))))

(with-decorator (app.route "/js/<path:path>")
  (fn [path] (send_from_directory "js" path)))

(with-decorator (app.route "/css/<path:path>")
  (fn [path] (send_from_directory "css" path)))

(with-decorator (app.route "/svg/<path:path>")
  (fn [path]
    (send_from_directory "svg" path)))

;(apply app.run [] {"debug" true})
;(app.run)
